
import stringcase

def execute(command_name, command_data):
    globals()[stringcase.snakecase(command_name)](command_data)

def start_benchmark(command_data):
    print 'Starting benchmark: ', str(command_data)

def stop_benchmark(command_data):
    print 'Stopping benchmark: ', str(command_data)

def start_metric_stream(command_data):
    print 'Starting metric stream: ', str(command_data)

def stop_metric_stream(command_data):
    print 'Stopping metric stream: ', str(command_data)



import config
import importlib
import sys
import pkg_resources
import benchmark
# from pkg_resources import iter_entry_points
# for entry_point in iter_entry_points(group='cms.plugin', name=None):
# print(entry_point)

def load_benchmarks():
    benchmarks = []

    bencmark_entry_points = \
        pkg_resources.iter_entry_points(group='dellve.benchmarks', name=None)

    for bencmark_entry_point in bencmark_entry_points:
        benchmark_class = bencmark_entry_point.load()

        if issubclass(benchmark_class, benchmark.Benchmark):
            benchmarks.append(benchmark_class)
        else:
            pass

            # TODO: add warning message saying that this 'benchmark' entry
            #       point is illformed and will be ignored!

    return benchmarks



import config
import executor
import gevent
import gevent.event
import gevent.pywsgi
import re
import falcon


class DELLveWorker(object):
    class BenchmarkStartRoute:

        url = '/benchmark/{bid}/start'

        def on_get(self, req, res, bid):
            res.status = falcon.HTTP_200
            res.content_type = 'application/json'
            res.body = '{"message": "Yo, let\'s start this benchmark # %d"}' % int(bid)
            # TODO: actually start this benchmark

    class BenchmarkStopRoute:

        url = '/benchmark/{bid}/stop'

        def on_get(self, req, res, bid):
            res.status = falcon.HTTP_200
            res.content_type = 'application/json'
            res.body = '{"message": "Yo, let\'s stop this benchmark # %d"}' % int(bid)
            # TODO: actually start this benchmark


    def __init__(self, port):
        # Create Falcon API
        api = falcon.API()

        # Add REST API routes
        for route in self._get_routes():
            api.add_route(route.url, route())

        # Create WSWGI server using gevent
        self._server = gevent.pywsgi.WSGIServer(('', port), api)

        # Create event (flag) to pend on
        self._stop = gevent.event.Event()

    def start(self):
        print 'Starting dellve worker ... OK'

        # Start server
        self._server.start()

        # Wait forever...
        self._stop.wait()

        # Stop server
        self._server.stop()

    def stop(self, *args):
        print 'Stopping dellve worker ... '
        self._stop.set()
        print '\bOK'

    @property
    def pidfile(self):
        return '.dellve/dellve.pid'

    @property
    def workdir(self):
        return '.dellve'

    @classmethod
    def _get_routes(cls):
        return map(lambda name: getattr(cls, name),
            [name for name in dir(cls) if name.endswith('Route')])



import dellve.loader
import pytest

class ConfigModuleStub:

    def __init__(self, data):
        self._data = {}

        self._data.update(data)


    def get(self, name):
        return self._data[name]

    def set(self, name, value):
        self._data[name] = value

test_data = {

}

def test_load_plugins():









import click
import config
import helper
import os
import service
import pick
import template

data_path = os.path.join(os.path.dirname(__file__), 'data')

@click.group()
@click.option('--config-file', 'config_file',
    default=os.path.join(data_path, 'config.yaml'),
    help='Configuration file name.', type=click.File('r'))
def cli(config_file):
    """
    DELLve benchmark command line interface.

    Type 'dellve COMMAND --help to see help for commands listed below.
    """
    config.load(config_file) # load DELLve configuration

@cli.command('ls', short_help='List installed benchmarks.')
def ls():
    """
    Lists installed DELLve benchmarks.
    """
    for benchmark in helper.load_benchmarks():
        print ' ', benchmark.name

@cli.command('start', short_help='Start the benchmark service.')
@click.option('--debug', default=False, is_flag=True, help='Debug mode.')
@click.option('--username', prompt=True)
@click.option('--password', prompt=True, hide_input=True)
def start(debug, username, password):
    """
    Starts DELLve benchmark background service.
    """
    click.echo('Starting benchmark service...')
    service.DELLveService(debug).start() # start DELLve daemon service

@cli.command('stop', short_help='Stop the benchmark service.')
def stop():
    """
    Stops DELLve benchmark background service.
    """
    click.echo('Stopping benchmark service...')
    service.DELLveService().stop()

@cli.command('status', short_help='Get the status of benchmark service.')
def status():
    """
    Gets the status DELLve benchmark background service.
    """
    service.DELLveService().status()

@cli.command('run', short_help='Run the benchmarks.')
@click.option('--all', '-A', 'run_all', default=False, is_flag=True,
    help='Run all benchmarks.')
def run(run_all):
    """Runs the user specified benchmarks"""

    # Load benchmarks into {'name': class} dictionary
    benchmarks = {b.name: b for b in helper.load_benchmarks()}

    if len(benchmarks) < 1: # ensure there're some benchmarks to be run
        click.echo('Please, install at least one benchmark plugin.', err=True)
        return

    # Note: user may select to run all benchmarks with --all/-A flag;
    #       if -all/-A flag is set, we run every single one of the 'benchmarks';
    #       otherwise, we prompt user to select a subset of the 'benchmarks'.

    if not run_all:
        options = benchmarks.keys()
        title = '\n'.join([
            'Please select benchmarks to run:',
            '',
            'Press UP and DOWN arrow keys to navigate',
            'Press SPACE to select benchmarks',
            'Press ENTER to proceede'
        ])

        # Ask user to pick benchmarks that he/she/ze wants to run
        picked = pick.pick(options, title, indicator='+', multi_select=True)

        # Note: pick() returns list of (name, index) pairs; since we only care
        #       about names in this list, we remove indexes from it below
        selected = {name for name, index in picked}

        # Filter out benchmarks that weren't selected
        benchmarks = filter(lambda k,v: k in selected, benchmarks.iteritems())

    for benchmark_name,  in selected:
        benchmarks[benchmark_name]().run()


@cli.command('new', short_help='Create a new benchmark.')
def new():
    """Generates the boilerplate code for a new DELLve-benchmark plugin"""
    dir_name = click.prompt('Please enter a directory name', default='./')
    package_name = click.prompt('Please enter a Python package name')
    benchmark_name = click.prompt('Please enter a unique benchmark class name')
    template.Template().render(dir_name, package_name, benchmark_name)

if __name__ == '__main__':
    cli()




import jinja2
import os


class Template(object):
    def __init__(self):
        data_path = os.path.join(os.path.dirname(__file__), 'data')

        with open(os.path.join(data_path, 'jinja2/setup.py.j2'), 'r') as f:
            self._setup_py_template = jinja2.Template(f.read())
        with open(os.path.join(data_path, 'jinja2/module.py.j2'), 'r') as f:
            self._module_py_template = jinja2.Template(f.read())
        with open(os.path.join(data_path, 'jinja2/__init__.py.j2'), 'r') as f:
            self._init_py_template = jinja2.Template(f.read())

    def render(self, dir_name, package_name, benchmark_name):
        dir_save = os.getcwd()

        package_dir = os.path.join(dir_name, package_name)
        if not os.path.exists(package_dir):
            os.makedirs(package_dir)
        os.chdir(package_dir)

        with open('setup.py', 'w') as f:
            f.write(self._setup_py_template.\
                render( package_name=package_name,
                        benchmark_name=benchmark_name ))

        package_source_dir = os.path.join('./', package_name)
        if not os.path.exists(package_source_dir):
            os.makedirs(package_source_dir)
        os.chdir(package_source_dir)

        with open('__init__.py', 'w') as f:
            f.write(self._init_py_template.\
                render( package_name=package_name,
                        benchmark_name=benchmark_name ))

        with open('benchmark.py', 'w') as f:
            f.write(self._module_py_template.\
                render( package_name=package_name,
                        benchmark_name=benchmark_name ))

        os.chdir(dir_save)



import dellve.config
import json, yaml
import os
import pytest
import tempfile

def test_load_json():
    """
    @brief      Tests dellve.config.load_json function.

    @param        JSON config file fixture.
    """
    file_handle, file_name = tempfile.mkstemp(suffix=".json")
    with open(file_name, 'w') as file:
        json.dump({'key': 'value'}, file)
    with open(file_name, 'r') as file:
        dellve.config.load_json(file)
        assert dellve.config.get('key') == 'value'
    os.remove(file_name)

def test_load_yaml():
    """
    @brief      Tests dellve.config.load_yaml function.

    @param      json_file  YAML config file fixture.
    """
    file_handle, file_name = tempfile.mkstemp(suffix=".yaml")
    with open(file_name, 'w') as file:
        yaml.dump({'key': 'value'}, file)
    with open(file_name, 'r') as file:
        dellve.config.load_yaml(file)
        assert dellve.config.get('key') == 'value'
    os.remove(file_name)

def test_load_generic_json():
    """
    @brief      Tests dellve.config.load with JSON config file.

    @param      json_file  JSON config file fixture.
    @param      yaml_file  YAML config file fixture.
    """
    file_handle, file_name = tempfile.mkstemp(suffix=".json")
    with open(file_name, 'w') as file:
        json.dump({'key': 'value'}, file)
    with open(file_name, 'r') as file:
        dellve.config.load(file)
        assert dellve.config.get('key') == 'value'
    os.remove(file_name)

def test_load_generic_yaml():
    """
    @brief      Tests dellve.config.load with YAML config file.

    @param      yaml_file  YAML config file fixture.
    """
    file_handle, file_name = tempfile.mkstemp(suffix=".yaml")
    with open(file_name, 'w') as file:
        json.dump({'key': 'value'}, file)
    with open(file_name, 'r') as file:
        dellve.config.load(file)
        assert dellve.config.get('key') == 'value'





from setuptools import setup, find_packages

setup(
    name='dellve',
    version='0.1.1',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'dellve': [
            'data/jinja2/*',
            'data/config.yaml'
        ]
    },
    install_requires=[
        'click',
        'daemonocle',
        'falcon',
        'gevent',
        'jinja2',
        'pick',
        'pyyaml',
        'stringcase'
    ],
    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest'
    ],
    entry_points='''
        [console_scripts]
        dellve=dellve.dellve:cli
    '''
)



import config
import daemonocle as daemon
import os
import worker

class DELLveService(object):

    def __init__(self, debug=False):

        dameon_worker = worker.DELLveWorker(config.get('http-port'))

        if not os.path.exists(dameon_worker.workdir):
            os.makedirs(dameon_worker.workdir)

        dameon_config = {
            'worker':               dameon_worker.start,
            'shutdown_callback':    dameon_worker.stop,
            'pidfile':              dameon_worker.pidfile,
            'workdir':              dameon_worker.workdir,
            'detach':               debug == False,
        }

        self._daemon = daemon.Daemon(**dameon_config)

    def start(self):
        # daemonocle.Daemon.do_action
        self._daemon.do_action('start')

    def stop(self):
        self._daemon.do_action('stop')

    def status(self):
        self._daemon.do_action('status')



import os
import os.path
__config = {}
__cache = {}

def load(file):
    """
    @brief      Loads a configuration file with JSON or YAML encoding.

    @param      file  Configuration file object.
    """
    if file.name.endswith('.json'):
        load_json(file)
    elif file.name.endswith(('.yml', '.yaml')):
        load_yaml(file)
    else:
        raise IOError('Couldn\'t load configuration file: %s' % file.name)

def load_yaml(file):
    """
    @brief      Loads a YAML-encoded configuration file.

    @param      file  Configuration file object.

    @return     { description_of_the_return_value }
    """
    import yaml
    data = yaml.load(file)
    if data:
        __config.update(data)

    # Cache directory of configuration file
    __cache['config-file-dir'] = os.path.dirname(os.path.realpath(file.name))

def load_json(file):
    """
    @brief      Loads a JSON-encoded configuration file.

    @param      file  Configuration file object.

    @return     { description_of_the_return_value }
    """
    import json
    data = json.load(file)
    if data:
        __config.update(data)

    # Cache directory of configuration file
    __cache['config-file-dir'] = os.path.realpath(file.name)

def get(name):
    """
    @brief      Gets value of a configuration parameter.

    @param      name   Parameter's name

    @return     Parameter's value
    """
    return __config[name]

def get_path(name):
    path = get(name)
    if os.path.isabs(path):
        return path
    else:
        return os.path.join(__cache['config-file-dir'], path)


